const Promise  = require("promise");
const fs       = require("fs");
const express  = require("express");
const body     = require("body-parser");
const helmet   = require("helmet");
const utils    = require("./Utils");
const Captcha  = require("./Captcha");

var Router = function(port){

  this.port = port;
  this.app = express();
  this.app.use(helmet())
  this.app.enable('trust proxy');
  this.app.disable('x-powered-by');
  this.app.use(body.json());
  this.app.use(body.urlencoded({extended: true}));
  this.app.set('views', __dirname + '/../views');
  this.app.set('view engine', 'twig');
  this.app.use('/static', express.static(__dirname + '/../assets'))
  this.app.listen(this.port);
  this.init = () => {

    return new Promise(

      (resolve, reject) => {

        this.app.get('/', (req, res) => {

            res.render('index', {title: 'Homepage'})

        })

        this.app.get('/contact', (req, res) => {

            res.render('contact', {title: 'Contattaci'})

        })

        this.app.post('/send', (req, res) => {

          var email = req.body.email,
              subject = req.body.subj,
              name = req.body.name,
              captcha = req.body.captcha,
              text = req.body.text,
              type = req.body.type,
              telegram_user = req.body.telegram_username;

              console.log(email, subject, name, text, type, telegram_user)

          if(!email || !subject || !name || !captcha || !text){

            res.status(400);
            res.json({error: 'missing_parameter'});
            return;

          }

          type = ((type == true) ? 'email' : 'telegram');

          var c = new Captcha(utils.config.captcha_key, captcha);

          c.verify()
            .then(success => {

              switch(type){

                case "email":

                  return utils.sendEmail(email, subject, text, name);

                break;

                case "telegram":

                  if(telegram_user.charAt(0) != "@")
                    telegram_user = "@" + telegram_user
                  return utils.sendMessage(email, subject, text, name, telegram_user);

                break;

                default:

                  res.status(400);
                  res.json({error: "bad_type"});
                  return;

                break;

              }

            })

            .then(success => {

              return res.json(success);

            })

            .catch(err => {

              console.log(err)
              res.status(400);
              return res.json(err);

            })

        })

        return resolve(true)

      }

    )

  }

}

module.exports = Router;
